<?php

namespace Bbg\DotDatConverter\Entities;

use Bbg\DotDatConverter\Exceptions\RequiredFieldsMissingException;

class Field
{
    private string $name;
    private int $start;
    private int $length;
    private int $end;
    private bool $required = false;
    private ?string $type = null;
    private ?string $cast = null;
    private string $default = '';
    private mixed $value = null;
    private int $padType = STR_PAD_RIGHT;

    public function __construct(object $data)
    {
        if (isset($data->recordType) && !empty($data->recordType)) {
            $this->setName('RecordType');
            $this->setStart(1);
            $this->setDefault($data->recordType);
            $this->setRequired(true);
        }

        if (isset($data->name)) {
            $this->setName($data->name);
        }

        if (isset($data->start)) {
            $this->setStart($data->start);
        }

        if (isset($data->length)) {
            $this->setLength($data->length);
        }

        if (isset($data->start) && isset($data->length)) {
            $this->setEnd(($data->start + $data->length));
        }

        if (isset($data->type)) {
            $this->setType($data->type);
        }

        if (isset($data->default)) {
            $this->setDefault($data->default);
        }

        if (isset($data->required)) {
            $this->setRequired($data->required);
        }
    }

    public function __toString(): string
    {
        return $this->toString();
    }

    public function toString(): string
    {
        $value = (is_null($this->value) ? $this->default : $this->value);

        if ($this->required && strlen($value) === 0) {
            throw new RequiredFieldsMissingException('Required field "' . $this->getName() . '" missing');
        }

        return substr(str_pad($value, $this->length, chr(32), $this->padType), 0, $this->length);
    }

    /**
     * @return int
     */
    public function getPadType(): int
    {
        return $this->padType;
    }

    /**
     * @param int $padType
     */
    public function setPadType(int $padType): self
    {
        if (in_array($padType, [STR_PAD_LEFT, STR_PAD_RIGHT, STR_PAD_BOTH])) {
            $this->padType = $padType;
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getCast(): ?string
    {
        return $this->cast;
    }

    /**
     * @return string
     */
    public function getType(): ?string
    {
        return $this->type;
    }

    /**
     * @return int
     */
    public function getEnd(): int
    {
        return $this->end;
    }

    /**
     * @return int
     */
    public function getLength(): int
    {
        return $this->length;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @return int
     */
    public function getStart(): int
    {
        return $this->start;
    }

    /**
     * @return string
     */
    public function getDefault(): string
    {
        return $this->default;
    }

    /**
     * @return bool
     */
    public function hasDefault(): bool
    {
        return (strlen($this->default) > 0);
    }

    /**
     * @return mixed
     */
    public function getValue(): mixed
    {
        return $this->value;
    }

    /**
     * @return bool
     */
    public function isRequired(): bool
    {
        return $this->required;
    }

    /**
     * @param string $cast
     */
    public function setCast(string $cast): self
    {
        $this->cast = $cast;
        return $this;
    }

    /**
     * @param int $end
     */
    public function setEnd(int $end): self
    {
        $this->end = $end;
        return $this;
    }

    /**
     * @param int $length
     */
    public function setLength(int $length): self
    {
        $this->length = $length;
        return $this;
    }

    /**
     * @param string $name
     */
    public function setName(string $name): self
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @param bool $required
     */
    public function setRequired(bool $required): self
    {
        $this->required = $required;
        return $this;
    }

    /**
     * @param int $start
     */
    public function setStart(int $start): self
    {
        $this->start = $start;
        return $this;
    }

    /**
     * @param string $type
     */
    public function setType(string $type): self
    {
        $this->type = $type;
        return $this;
    }

    /**
     * @param string $default
     */
    public function setDefault(string $default): self
    {
        $this->default = $default;
        return $this;
    }

    /**
     * @param mixed $value
     */
    public function setValue(mixed $value): self
    {
        $this->value = $value;
        return $this;
    }
}